///////////////////////////////////////////////////////////////////////////
// Copyright © 2014 Esri. All Rights Reserved.
//
// Licensed under the Apache License Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
///////////////////////////////////////////////////////////////////////////
/*global define, document */
define(['dojo/_base/declare',
    'dijit/_WidgetBase',
    'dojo/_base/lang',
    'dojox/gfx',
    'dojo/on',
    'dojo/mouse',
    'dojo/dom-construct',
    'dojo/dom-attr',
    'dojo/_base/array',
    'dojo/dom',
    'dojo/query',
    'dojo/dom-class',
    'dojo/dom-style',
    'dojo/Evented',
    'esri/symbols/jsonUtils'
  ],
  function(declare,
    _WidgetBase,
    lang,
    gfx,
    on,
    mouse,
    domConstruct,
    domAttr,
    array,
    dom,
    query,
    domClass,
    domStyle,
    Evented,
    jsonUtils) {
    return declare([_WidgetBase, Evented], {

      'class': 'widgets-nhnearme-list',
      _itemCSS: "nhnearme-search-list-item",
      _itemSelectedCSS: "nhnearme-search-list-item selected",
      _itemAltCSS: "nhnearme-search-list-item alt",
      DEBUG:false,

      startup: function() {
        this.items = [];
        this.selectedIndex = -1;
        this._selectedNode = null;
        this._listContainer = domConstruct.create("div");
        domClass.add(this._listContainer, "nhnearme-search-list-container");
        this.own(on(this._listContainer, "click", lang.hitch(this, this._onClick)));
        domConstruct.place(this._listContainer, this.domNode);
      },
      
      log: function(messageString){
        if(this.DEBUG){
          console.log(messageString);
        }
      },

      add: function(item) {
        if (arguments.length === 0) {
          return;
        }
        this.items.push(item);
        var div = domConstruct.create("div");
        domAttr.set(div, "id", this.id.toLowerCase()+item.id);
        this.own(on(div, mouse.enter, lang.hitch(this, this._onMouseOver)));//added mouse enter per item
        this.own(on(div, mouse.leave, lang.hitch(this, this._onMouseLeave)));//added mouse enter per item

        var iconDiv = domConstruct.create("div");
        domAttr.set(iconDiv, "id", this.id.toLowerCase()+item.id);
        domClass.add(iconDiv, "iconDiv");
        domConstruct.place(iconDiv, div);

        var removeDiv = domConstruct.create('div');
        domConstruct.place(removeDiv, div);
        domClass.add(removeDiv, 'removediv');
        domAttr.set(removeDiv, 'id', this.id.toLowerCase()+item.id);

        var removeDivImg = domConstruct.create('div');
        domClass.add(removeDivImg, 'removedivImg');
        domConstruct.place(removeDivImg, removeDiv);
        domAttr.set(removeDivImg, 'id', this.id.toLowerCase()+item.id);
        domAttr.set(removeDivImg, 'title', item.removeResultMsg);
        this.own(on(removeDivImg, 'click', lang.hitch(this, this._onRemove)));

        var rTitle = domConstruct.create("strong");
        domAttr.set(rTitle, "id", this.id.toLowerCase()+item.id);
        domClass.add(rTitle, "label");
        rTitle.textContent = rTitle.innerText = item.title;
        domConstruct.place(rTitle, div);
        if(item.alt){
          domClass.add(div, this._itemAltCSS);
        }else{
          domClass.add(div, this._itemCSS);
        }
        


        if(item.rsltcontent !== ""){
          var attArr = item.rsltcontent.split('<br>');
          var attValArr, tHasColor, bIndex, eIndex, tColor, vHasColor, vColor;
          var label, attTitle, attVal;
          var arrayLength = attArr.length;
          for (var i = 0; i < arrayLength; i++) {
            attValArr = attArr[i].split(': ');
            attTitle = domConstruct.create('font');
            domAttr.set(attTitle, 'id', this.id.toLowerCase()+item.id);
            if(attValArr[0].toLowerCase().indexOf('<em>') > -1){
              domStyle.set(attTitle, 'font-style', 'italic');
            }
            if(attValArr[0].toLowerCase().indexOf('<strong>') > -1){
              domStyle.set(attTitle, 'font-weight', 'bold');
            }
            if(attValArr[0].toLowerCase().indexOf('<u>') > -1){
              domStyle.set(attTitle, 'text-decoration', 'underline');
            }
            tHasColor = (attValArr[0].toLowerCase().indexOf("<font color='") > -1)?true:false;
            if(tHasColor){
              bIndex = attValArr[0].toLowerCase().indexOf("<font color='") + 13;
              eIndex = attValArr[0].toLowerCase().indexOf("'>", bIndex);
              tColor = attValArr[0].substr(bIndex, eIndex - bIndex);
              domStyle.set(attTitle, 'color', tColor);
            }

            attTitle.textContent = attTitle.innerText = attValArr[0].replace(/<[\/]{0,1}(em|EM|strong|STRONG|font|FONT|u|U)[^><]*>/g, "");
            label = domConstruct.create('p');
            domAttr.set(label, 'id', this.id.toLowerCase()+item.id);
            domClass.add(label, 'label');
            attVal = domConstruct.create('font');

            if(attValArr[1].toLowerCase().indexOf('<em>') > -1){
              domStyle.set(attVal, 'font-style', 'italic');
            }
            if(attValArr[1].toLowerCase().indexOf('<strong>') > -1){
              domStyle.set(attVal, 'font-weight', 'bold');
            }
            if(attValArr[1].toLowerCase().indexOf('<u>') > -1){
              domStyle.set(attVal, 'text-decoration', 'underline');
            }
            vHasColor = (attValArr[1].toLowerCase().indexOf("<font color='") > -1)?true:false;
            if(vHasColor){
              bIndex = attValArr[1].toLowerCase().indexOf("<font color='") + 13;
              eIndex = attValArr[1].toLowerCase().indexOf("'>", bIndex);
              vColor = attValArr[1].substr(bIndex, eIndex - bIndex);
              domStyle.set(attVal, 'color', vColor);
            }

            if (attValArr[1] === 'null') {
              attVal.textContent = attVal.innerText = ": ";
            } else {
              attVal.textContent = attVal.innerText = ": " + attValArr[1].replace(/<[\/]{0,1}(em|EM|strong|STRONG|font|FONT|u|U)[^><]*>/g, "");
            }
            domConstruct.place(attTitle, label);
            domConstruct.place(attVal, label);
            domConstruct.place(label, div);
          }
        }else{
          var label2 = domConstruct.create("p");
          domClass.add(label2, "label");
          label2.textContent = label2.innerText = " ";
          domConstruct.place(label2, div);
        }
        if(document.all && !document.addEventListener){
          //do nothing because it is IE8
          //And I can not produce swatches in IE8
        }else{
          var mySurface = gfx.createSurface(iconDiv, 40, 40);
          var descriptors = jsonUtils.getShapeDescriptors(item.sym);
          if(descriptors.defaultShape){
            var shape = mySurface.createShape(descriptors.defaultShape).setFill(descriptors.fill).setStroke(descriptors.stroke);
            shape.applyTransform({ dx: 20, dy: 20 });
          }
        }

        if(item.links && item.links.length > 0){
          var linksDiv = domConstruct.create("div");
          domConstruct.place(linksDiv, div);
          domClass.add(linksDiv, 'linksdiv');
        }
        
        array.forEach(item.links, function(link){
          if(link.popuptype === "text"){
            var linkText = domConstruct.toDom("<p><a href='" + link.link + "' target='_blank' title='" + link.alias + "'>" + link.alias + "</a></p>");
            domConstruct.place(linkText, linksDiv, 'before');
            domClass.add(linkText, 'labellink');
          }else{
            var linkImg = domConstruct.toDom("<a href='" + link.link + "' target='_blank' title='" + link.alias + "'><img src='" + link.icon + "' alt='" + link.alias + "' border='0' width='20px' height='20px'></a>");
            domConstruct.place(linkImg, linksDiv);
            domClass.add(linkImg, 'linkIcon');
          }
        });
        domConstruct.place(div, this._listContainer);
        this.clearSelection();
      },

      remove: function(index) {
        var item = this.items[index];
        domConstruct.destroy(this.id.toLowerCase() + item.id + "");
        this.items.splice(index, 1);
        if (this.items.length === 0) {
          this._init();
        }
        if(item.id === this._selectedNode){
          this._selectedNode = null;
        }
        this.clearSelection();
      },

      _init: function() {
        this.selectedIndex = -1;
        this._selectedNode = null;
      },

      clear: function() {
        if(typeof this.items === 'undefined' ) return;
        this.items.length = 0;
        this._listContainer.innerHTML = "";
        this._init();
      },
      
      _onMouseLeave: function(evt) {
        this.log("ms-list-leaver");
        var item = this._getTargetOfEventItem(evt);
        if(item) this.emit('list-mouse-leave', this.selectedIndex, item);//can add leave if needed
      },
      
      _onMouseOver: function(evt) {
        this.log("ms-list-over");
        var item = this._getTargetOfEventItem(evt);
        if(item) this.emit('list-mouse-over', this.selectedIndex, item);
      },

      _onClick: function(evt) {
        this.log("ms-click");
        var item = this._getTargetOfEventItem(evt);
        if(item)this.emit('click', this.selectedIndex, item);
      },
      
      _getTargetOfEventItem: function(evt){
        if (evt.target.id === "" && evt.target.parentNode.id === "") {
          return null;
        }
        var id = evt.target.id.toLowerCase();
        if (!id) {
          id = evt.target.parentNode.id;
        }
        var item = this._getItemById(id);
        if (!item) {
          return null;
        }
        domClass.replace(id, this._itemSelectedCSS, ((item.alt) ? this._itemAltCSS:this._itemCSS));
        if (this._selectedNode) {
          var item_selected = this._getItemById(this._selectedNode);
          if(item_selected){
            domClass.replace(this._selectedNode, ((item_selected.alt)? this._itemAltCSS:this._itemCSS), this._itemSelectedCSS);
          }
        }
        this._selectedNode = id;
        return item;
      },

      _onRemove: function(evt) {
        evt.stopPropagation();

        var id = evt.target.id.toLowerCase();
        if (!id) {
          id = evt.target.parentNode.id;
        }
        var item = this._getItemById(id);
        if (!item) {
          return;
        }
        this._selectedNode = id;
        this.emit('remove', this.selectedIndex, item);
      },

      _getItemById: function(id) {
        id = id.replace(this.id.toLowerCase(),"");
        var len = this.items.length;
        var item;
        for (var i = 0; i < len; i++) {
          item = this.items[i];
          if (item.id === id) {
            this.selectedIndex = i;
            return item;
          }
        }
        return null;
      },

      clearSelection: function () {
        this._selectedNode = null;
        this.selectedIndex = -1;
        query('.nhnearme-search-list-item').forEach(function(node){
          domClass.remove(node, "alt");
        });
        array.map(this.items, lang.hitch(this, function(item, index){
          item.alt = (index % 2 === 0);
          if(item.alt){
            domClass.add(this.id.toLowerCase() + item.id + "", "alt");
          }
        }));
      },

      setSelectedItem: function(id) {
        var item = this._getItemById(id);
        if (!item) {
          return;
        }
        domClass.replace(id, this._itemSelectedCSS, ((item.alt) ? this._itemAltCSS:this._itemCSS));
        if (this._selectedNode) {
          var item_selected = this._getItemById(this._selectedNode);
          if(item_selected){
            domClass.replace(this._selectedNode, ((item_selected.alt)? this._itemAltCSS:this._itemCSS), this._itemSelectedCSS);
          }
        }
        this._selectedNode = id;
      }
    });
  });
