///////////////////////////////////////////////////////////////////////////
// Copyright © 2014 Esri. All Rights Reserved.
//
// Licensed under the Apache License Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
///////////////////////////////////////////////////////////////////////////

define(['dojo/_base/declare',
    'dojo/_base/html',
    './NHNearMeList',
    'dojo/query',
    'dojo/_base/lang',
    'dojo/dom',
    'dojo/dom-class',
    'dojo/dom-construct',
    'dojo/_base/array',
    'dojo/Deferred',
    'dojo/promise/all',
    'dojo/dom-attr',
    'dojo/string',
    'dojo/date/locale',
    'dojo/on',
    'dijit/_WidgetsInTemplateMixin',
    'jimu/BaseWidget',
    'dijit/form/HorizontalSlider',
    'dijit/form/TextBox',
    'esri/dijit/InfoWindow',
    'esri/dijit/PopupTemplate',
    'esri/geometry/Extent',
    'esri/geometry/geometryEngine',   
    'esri/geometry/webMercatorUtils',
    'esri/tasks/query',
    'esri/tasks/QueryTask',
    'esri/layers/FeatureLayer',
    'esri/geometry/Polygon',
    'esri/layers/GraphicsLayer',
    'esri/graphic',
    'esri/symbols/jsonUtils',
    'esri/symbols/PictureMarkerSymbol',
    'esri/dijit/Search',
    'esri/tasks/locator',
    'esri/request'

  ],
  function(declare,html,List,dojoQuery,lang,dom,domClass,domConstruct,array,Deferred,all,domAttr,string,locale, on,_WidgetsInTemplateMixin,BaseWidget, 
            HorizontalSlider,TextBox,InfoWindow,PopupTemplate,Extent,geometryEngine,webMercatorUtils,Query,QueryTask,FeatureLayer,Polygon,
            GraphicsLayer,Graphic,symbolJsonUtils,PictureMarkerSymbol,SearchWidget,Locator,esriRequest ) {
    var clazz = declare([BaseWidget, _WidgetsInTemplateMixin], {
      baseClass: 'jimu-widget-nhnearme',

      m_iTimerTicks:0,
      m_lastQueryTime:new Date(),
      featureServiceFailures: [],
      resultLayersInfos: [],
      DEBUG:false,
      nearBySearchLayers:[],
      searchWgt:null,
      resultFormatString:"",
      lastKnownGeometry:null,
			RADIUS_OF_EARTH_IN_MILES:3963,
			RADIUS_OF_EARTH_IN_FEET:20925525,
			RADIUS_OF_EARTH_IN_KM:6378,
			RADIUS_OF_EARTH_IN_M:6378000,
      highlightGraphicsLayer: null, 
      locatorInstance: null, // Locator instance to reverse geocode the address
      lastLocatorSearchedMapPoint:null,
      searchCenterPointGeo:null,
      allFeaturesFound:[],
      repositionOnListItem: null,
      layersWereRemoved: false,
      //TODO's
      
      postCreate: function() {
        this.resultFormatString = '<font><em>[attribname]</em></font>: <font>[attribvalue]</font><br>';
        this.divResultMessage.textContent = "";
        this.removeLayersOnClearCbx.setValue(this.config.clearOnClose || false);
        this.clearOnCloseCbx.setValue(this.config.clearOnClose || true);
        this.repositionOnListItem = (typeof this.config.repositiononlistitemhover === 'undefined') ? false : this.config.repositiononlistitemhover;
        //this.googleDirectionsCbx.setValue(false);//used in  could use here as opposite direction.
        this._bufferGraphicLayer = new GraphicsLayer({id:'NHNearMeBuffer'});
        this.map.addLayer(this._bufferGraphicLayer);
        this.highlightGraphicsLayer = new GraphicsLayer({id:'NHNearMeBufferBullseye'});//work around for bug related to highlightSymbol, should be able to hand symbol as option
        this.map.addLayer(this.highlightGraphicsLayer);
        this.getLayerInfosFromServer();
        this.log('NHNearMe-postCreate');
      },
      
      startup: function(){
        this.inherited(arguments);
        this.log('NHNearMe-Startup fetchDataByName');
      },
      
      getLayerInfosFromServer: function () {
        this.nhNearMeProgressLabel.textContent = "(intitialzing please wait...)";
        var len = this.config.layers.length;
        if (len > 0) {
          var defs = array.map(this.config.layers, lang.hitch(this, function (layerConfig) {
              return this.createLayerRequestDefered(layerConfig.url);
          }));
          this.featureServiceFailures = [];
          all(defs).then(lang.hitch(this, function (results) {
            array.forEach(results, lang.hitch(this, function (result, j) {
              if(result.state === 'success'){
                var layerInfo = result.value;
                var layerConfig = this.config.layers[j];
                layerConfig.objectIdField = "OBJECTID";//we always publish with this objectid name
                layerConfig.existObjectId = true;
                layerConfig.typeIdField = layerInfo.typeIdField;
                if (!layerInfo.drawingInfo) {
                  layerInfo.drawingInfo = {};
                }
                layerInfo.name = this.nls.search + ' ' + this.nls.results + ': ' + layerConfig.name;//TODO is this what we want for name?
                layerInfo._titleForLegend = layerInfo.name;
                layerInfo.minScale = 0;
                layerInfo.maxScale = 0;
                layerInfo.effectiveMinScale = 0;
                layerInfo.effectiveMaxScale = 0;
                layerInfo.defaultVisibility = true;
                this.resultLayersInfos.push(layerInfo);
                this.log("LAYERINFO-LOADED-FOR:" + layerInfo.name);
                newLayer = this.createNearBySearchLayer(this.resultLayersInfos.length-1);
                newLayer._nearByLayerIndex = this.resultLayersInfos.length-1;
                newLayer._nearByLayerUrl = layerConfig.url;
                newLayer._nearByLayerName = layerConfig.name;
                newLayer._nearByLayerRevGeocode = layerConfig.reversegeocode;
                
                if(newLayer){
                  this.nearBySearchLayers.push(newLayer);
                  this.map.addLayer(newLayer);
                  this.log("LAYER-Added-To-Map:" + layerInfo.name);
                }
                this.nearBySearchLayers[this.nearBySearchLayers.length - 1].on("mouse-down", lang.hitch(this, this.featureGraphicMouseClickHandler));
              }else{
                this.log("LAYER-FAILED:" + this.config.layers[j].name);
                this.featureServiceFailures.push(this.config.layers[j].name);
                this.resultLayersInfos.push({});
              }
            }));
      
            if(this.featureServiceFailures.length > 0){
              console.info("failed-to-load-feature-services:", this.featureServiceFailures);
            }
            //init widget user controls after service info is successfully loaded.
            //this.nearBySearchLayers[0].on("mouse-over", lang.hitch(this, this.featureGraphicMouseOverHandler));//for labels popup up.

            this.createSlider();//initialize widget slider controls

            swOptions = {
              map: this.map,
              addLayersFromMap: false,
              autoNavigate: false,
              autoComplete: true,
              enableInfoWindow:false,
              minCharacters: 0,
              maxLocations: 5,
              searchDelay: 100,
              enableHighlight: false
            };

            this.searchWgt = new SearchWidget(swOptions, domConstruct.create("div", { "class": "searchControl" }, this.search));//attach search widget to search dojo attachpoint of widget.html
            //this.searchWgt.set(highlightSymbol,symbol);
            defaultSources = [];
            var geocoderURL = this.config.geocoderurl;
            var geocoder={};
            geocoder.locator = new Locator(geocoderURL);
            geocoder.suggest = true;
            geocoder.placefinding = false;
            geocoder.singleLineFieldName = "SingleLine";
            geocoder.name = "NH911";
            defaultSources.push(geocoder);
            
            //this.initReverseGeocoder();
            //this.own(on(this.map.infoWindow, 'show', lang.hitch(this, '_onMapInfoWindowShow')));

            this.own(this.list.on("list-mouse-over", lang.hitch(this, this.listItemMouseOver)));//can catch other custom events
            //this.own(this.list.on("list-mouse-leave", lang.hitch(this, this.listItemMouseLeave)));//can catch other custom alvnearmelis events

            this.searchWgt.set("sources", defaultSources);
            this.own(this.searchWgt.on("select-result", lang.hitch(this, this._selectResult)));
            //this.own(this.searchWgt.on("clear-search", lang.hitch(this, this.clear)));
            this.searchWgt.startup();
            this.nhNearMeProgressLabel.textContent = "";
            this.fetchDataByName('Search');//check if we have messages pending from map search widget
          }), lang.hitch(this, function (err) {
            console.error(err);
            this.nhNearMeProgressLabel.textContent = "(errors)";
            for (var j = 0; j < this.config.layers.length; j++) {
              var layer = new GraphicsLayer();
              this.resultLayersInfos.push(layer);
            }
          }));
        }
      },

      /**
      * This function initialize the Locator widget for reverse geocoding
      * @memberOf widgets/NHNearMe/Widget
      */
      initReverseGeocoder: function () {
        var geocodeURL = "https://geocode.arcgis.com/arcgis/rest/services/World/GeocodeServer";
        var locatorName = "ESRI"
        if(this.config.geocoderurl){
          geocodeURL = this.config.geocoderurl;
          locatorName = "NH911";
        }
        this.locatorInstance = new Locator(geocodeURL);
        this.locatorInstance.locatorName = locatorName;
      },
      
      _onMapInfoWindowShow: function(evt){
        this.log("_onMapInfoWindowShow");
        if (evt.target.features[0]._performRevGeode) {
          this._revGeocodeGraphic(evt);
        }
      },
      
      _revGeocodeGraphic: function (evt){
        if(evt.target.features[0]){
          this.lastLocatorSearchedMapPoint = evt.target.features[0].geometry;
          this.locatorInstance.locationToAddress(webMercatorUtils.webMercatorToGeographic(this.lastLocatorSearchedMapPoint), this.geocoderDistance,
            lang.hitch(this, this.onLocationToAddressComplete),
            lang.hitch(this, this.onLocationToAddressInComplete)
          );
        }
        this.log("RevGeocode");
      },
      
      /**
      * Callback handler called once location is reverse geocoded
      * @memberOf 
      **/
      onLocationToAddressComplete: function (result) {
        var messageStr="NH911 ADDRESS:<br>";
        
        var latLonStr = this.getLastLatLonAsString();
        if (result.address) {
          //this.map.infoWindow.clearFeatures();
          if(this.locatorInstance.locatorName === "NH911"){
            messageStr+= result.address.Street + ", " + result.address.City;
          }else{
            messageStr="ADDRESS:<br>" + result.address.Match_addr;
          }
          
          this.map.infoWindow.setTitle(messageStr);
          return;
          //or place it in the content first or last.
          var infoWindowContentPaneNode = dojoQuery(".contentPane", this.map.infoWindow.domNode)[0];
          if(infoWindowContentPaneNode){
            dojo.create("div", {style: "color:#FF0000", innerHTML: messageStr },infoWindowContentPaneNode,"first");
          }
        }
      },
      
      getLastLatLonAsString: function(){
        var geoPt = webMercatorUtils.webMercatorToGeographic(this.lastLocatorSearchedMapPoint);
        var lon = (geoPt.x).toPrecision(6);
        var lat = (geoPt.y).toPrecision(6);
        var latLonStr="";
        latLonStr =  lat + "," + lon;
        return latLonStr;
      },

      onLocationToAddressInComplete: function (result) {
        this.log("geocodeincomplete");
      },      
      
      clearButtonClick: function(){
        this.clearNearbyLayerGraphics();
        this.clear(this.removeLayersOnClearCbx.getValue());//if true it removes layers from map which is slow process the more layers you have
      },
      
      onClose: function () {
        if(this.clearOnCloseCbx.getValue()){
          this.hideInfoWindow();
          this.map.infoWindow.clearFeatures();

          if (this.nearBySearchLayers.length > 0) {
            this.nearBySearchLayers.forEach(
              function(lyr){
                lyr.hide();
              }
            );
          }
          if(this.highlightGraphicsLayer){
            this.highlightGraphicsLayer.hide();
          }
          if (this._bufferGraphicLayer) {
            this._bufferGraphicLayer.hide();
          }
        }
      },
      
      onOpen: function(){
        if(this.list.items.length > 0){
          if (this.nearBySearchLayers.length > 0) {
            this.nearBySearchLayers.forEach(
              function(lyr){
                lyr.show();
              }
            );
          }
          if(this.highlightGraphicsLayer){
            this.highlightGraphicsLayer.show();
          }
          if (this._bufferGraphicLayer) {
            this._bufferGraphicLayer.show();
          }
        }
      },


      clear: function (removeLyrs) {
        this.hideInfoWindow();
        this.divResultMessage.textContent = "Clearing Features Please wait...";
        this.map.infoWindow.clearFeatures();
        this.list.clear();
        this.divResultMessage.textContent="";
        if(this.highlightGraphicsLayer){
          this.highlightGraphicsLayer.clear();
        }
        if (this._bufferGraphicLayer) {
          this._bufferGraphicLayer.clear();
        }
        if(removeLyrs){
          this.manageNearbyLayersOnMap(false);
        }
        return false;
      },

      manageNearbyLayersOnMap: function (add) {
        var i = 0, lyrCnt =  this.nearBySearchLayers.length;
        if (lyrCnt > 0) {
          var aLyr;
          for (i = 0; i < lyrCnt; i++) {
            aLyr = this.nearBySearchLayers[i];
            if(add){
              this.layersWereRemoved = false;
              this.map.addLayer(aLyr);
              this.log("layer-type-added:" + aLyr.type + " name:" + aLyr.name)
            }else{
              this.divResultMessage.textContent = "Removing Layer: " + aLyr.name;
              this.layersWereRemoved = true;
              this.map.removeLayer(aLyr);
              this.log("layer-type-removed:" + aLyr.type + " name:" + aLyr.name)
            }
          }
          if(!add) this.divResultMessage.textContent = lyrCnt + " Layers Removed";
        }
      },
      
      clearNearbyLayerGraphics: function(){
        this.divResultMessage.textContent = "Clearing Layers Please wait...";
        if (this.nearBySearchLayers.length > 0) {
          this.nearBySearchLayers.forEach(lang.hitch(this,
            function(lyr){
              lyr.clear();
              this.divResultMessage.textContent = "Cleared Layer " + lyr.name;
            }
          ));
        }
      },
      
      hideInfoWindow: function () {
        if (this.map && this.map.infoWindow) {
          this.map.infoWindow.hide();
        }
      },
      
      listItemMouseOver: function(index, item){
        this.log("listItemMouseOver");//center no zoom, click will zoom
        if(this.map.infoWindow.isShowing) this.map.infoWindow.hide();
        
        if (this.repositionOnListItem) {
          this.map.centerAt(item.centerpoint).then(lang.hitch(this, function(){
            if (this.map.infoWindow) {
              this.map.infoWindow.setFeatures([item.graphic]);
              if (this.map.infoWindow.reposition) {
                this.map.infoWindow.reposition();
              }
            }
          }));
        }else{
            this.map.infoWindow.setFeatures([item.graphic]);
        }
      },
      
      listItemMouseLeave: function(index, item){
        this.log("listItemMouseLeave");//center no zoom, click will zoom
      },
      
			googleDirections: function(somePoint) {
				var somePointWorldGeo = webMercatorUtils.webMercatorToGeographic(somePoint);
				var locationWorldGeo = webMercatorUtils.webMercatorToGeographic(this.searchCenterPointGeo);
				var googleUrl = "https://maps.google.com/maps?";
				var saddrParam = "saddr=";
				var daddrParam = "daddr=";
				var fromCoord=somePointWorldGeo.y + "," + somePointWorldGeo.x;
				var toCoord=locationWorldGeo.y + "," + locationWorldGeo.x;
        var dirMode = "mode=driving";
				var linkURL = googleUrl + saddrParam + fromCoord + "&" + daddrParam + toCoord + "&" + dirMode; 
        var windowSettings = "location=yes,height=570,width=520,scrollbars=yes,status=yes";
        
				var win = window.open(linkURL, "_blank", windowSettings);
			},
      
      selectedResultListItem: function (index, item) {
        this.log("_selectResultItem");
        if(this.map.infoWindow.isShowing) this.map.infoWindow.hide();
        var zoomScale = this.config.zoomScale;
        var centerOnly = this.config.centerOnlyOnListItemSelect || true;
        if(centerOnly){
          this._centerOnItem(item);
        }else{
          if (this.map.getScale() > zoomScale) {
            this.map.setScale(zoomScale).then(this.map.centerAt(item.centerpoint).then(lang.hitch(this, function () {
              if (this.map.infoWindow) {
                this.map.infoWindow.setFeatures([item.graphic]);
                if (this.map.infoWindow.reposition) {
                  this.map.infoWindow.reposition();
                }
                this.map.infoWindow.show(item.centerpoint);
              }
            })));
            
          }else{
            this._centerOnItem(item);
          }
        }
//        if(this.googleDirectionsCbx.getValue() && this.searchCenterPointGeo){
//          this.googleDirections(item.centerpoint);
//        }
      },
      
      _centerOnItem: function (item){
          this.map.centerAt(item.centerpoint).then(lang.hitch(this, function () {
              if (this.map.infoWindow) {
                this.map.infoWindow.setFeatures([item.graphic]);
                if (this.map.infoWindow.reposition) {
                  this.map.infoWindow.reposition();
                }
                this.map.infoWindow.show(item.centerpoint);
              }
            }));
      },
      
      _selectResult: function (evt) {
        //code to focus out the textbox (issue with android tab)
        //this.search.blur();
        this.clear();
        if(evt.result){
          var geom = evt.result.feature.geometry;
          this.lastKnownGeometry = geom;
          this.highlightSelectedLocation(geom);
          this.log("geom-ll:" + geom.x + "," + geom.y);
          this.searchNearBy(geom);
        }
        this.log("select-result");
      },      
      
      createNearBySearchLayer: function (layerIndex) {
        var resultLayer = null;
        var layerConfig = this.config.layers[layerIndex];
        var clonedLayerInfo = lang.clone(this.resultLayersInfos[layerIndex]);
        var configuredFieldNames = this.getOutputFieldsPerConfig(layerIndex);
        //override cloned layer infos fields variable, with only configured fields
        clonedLayerInfo.fields = array.filter(clonedLayerInfo.fields, lang.hitch(this, function (fieldInfo) {
          return configuredFieldNames.indexOf(fieldInfo.name) >= 0;
        }));
        //always include
        var featureCollection = {
          layerDefinition: clonedLayerInfo,
          featureSet: null
        };
        resultLayer = new FeatureLayer(featureCollection);
        resultLayer.setInfoTemplate(this._configurePopupTemplateForNearByLayer(layerIndex));
        //resultLayer.on("mouse-out", lang.hitch(this, this.car_graphicMouseOutHandler));

        return resultLayer;
      },
      
			featureGraphicMouseOverHandler: function(event)
			{
        this.log("featureGraphicMouseOverHandler");
      },
      
			featureGraphicMouseClickHandler: function(event)
			{
        this.log("featureGraphicMouseClickHandler");
        this.map.infoWindow.hide();//hide any current open info
        if (this.map.infoWindow) {
          //event.graphic.setInfoTemplate(this._configurePopupTemplateForNearByLayer(event.graphic._layer.layerId));//override default
          this.map.infoWindow.setFeatures([event.graphic]);
          this.map.infoWindow.show(event.graphic.geometry);
        }
      },
      
      //use field infos from config, and label for popup as well
      _configurePopupTemplateForNearByLayer: function (layerIndex) {
        var len = this.config.layers.length;
        var fieldInfosArray = [];
        this.log("_configurePopupTemplateForNearByLayer:layer:" + layerIndex);
        var layerConfig = this.config.layers[layerIndex];
        var confFieldsArr = layerConfig.fields.field;
        var fieldsLen = confFieldsArr.length;
        for (var i = 0; i < fieldsLen; i++) {
          if(confFieldsArr[i].popup === false) continue;
          var fieldInfoObj = {
            fieldName: confFieldsArr[i].name,
            visible: true,
            label: confFieldsArr[i].label
          };
          fieldInfosArray.push(fieldInfoObj);
        }
        
        var mediaLinks = [];
        var pminfos = [];
        var popUpMediaInfo;
        if (layerConfig.links && layerConfig.links.link) {
          mediaLinks = layerConfig.links.link;
          for (var i = 0; i < mediaLinks.length; i++) {
            if (mediaLinks[i].iconUrl !== "" && mediaLinks[i].popuptype !== "text") {
              popUpMediaInfo = {};
              popUpMediaInfo.type = mediaLinks[i].popuptype;
              var val = {};
              val.sourceURL = mediaLinks[i].sourceUrl;
              val.linkURL = mediaLinks[i].linkUrl;
              popUpMediaInfo.value = val;
              popUpMediaInfo.caption = mediaLinks[i].caption;
            }else{
              //TODO handle text link types or other media types here.
            }
            pminfos.push(popUpMediaInfo);
          }
        }
        var popupTemplate = new PopupTemplate({
            fieldInfos: fieldInfosArray,
            mediaInfos: pminfos
          });
          return popupTemplate;
      },
      
      getOutputFieldsPerConfig: function (layerIndex) {
        var layerConfig = this.config.layers[layerIndex];
        var lcFields = layerConfig.fields.field;
        var outFieldNames = array.map(lcFields, lang.hitch(this, function (lcField) {
          return lcField.name;//get only field name from each field element
        }));
        var layersFieldInfos = this.resultLayersInfos[layerIndex].fields;
        var layersFieldNames = array.map(layersFieldInfos, lang.hitch(this, function (fieldInfo) {
          return fieldInfo.name;//get only field name from layer info fields
        }));
        outFieldNames = array.filter(outFieldNames, lang.hitch(this, function (fieldName) {
          return layersFieldNames.indexOf(fieldName) >= 0;//filter on config fieldName that exists in fieldInfo
        }));
        
        var objectIdField = layerConfig.objectIdField;
        if (array.indexOf(outFieldNames, objectIdField) < 0) {
          outFieldNames.push(objectIdField);
        }      
        return outFieldNames;
      },
      
      //creates a deferreds for esriRequest service calls for layer info of our configured layers
      createLayerRequestDefered: function (layerUrl) {
        var def = new Deferred();
        esriRequest({
          url: layerUrl,
          content: {
            f: 'json'
          },
          handleAs: 'json',
          callbackParamName: 'callback'
        }).then(lang.hitch(this, function (layerInfo) {
          def.resolve({state: 'success', value: layerInfo});
        }), lang.hitch(this, function (err) {
          def.resolve({state: 'failure', value: err});
        }));
        return def;
      },

      searchNearBy: function (geometry) {
        //will build an array of defereds from our nearbysearchlayers array
        this.lastKnownGeometry = geometry;
        if(this.layersWereRemoved){
          this.manageNearbyLayersOnMap(true);
        }
        this.allFeaturesFound = [];
        var i=0;
        var defs=[];
        for(i=0; i < this.nearBySearchLayers.length; i++){
          var queryParams = new Query();
          var distanceMiles = this._slider.value;
          var bufferGeom = geometryEngine.geodesicBuffer(geometry, distanceMiles, "miles");//9036 is km, see https://developers.arcgis.com/javascript/3/jsapi/esri.geometry.geometryengine-amd.html#buffer
          this.showBuffer([bufferGeom]);
          this.map.setExtent(bufferGeom.getExtent(),true);//zooms to the extent of our buffer poly
          if (geometry) {
            queryParams.geometry = bufferGeom;
            queryParams.spatialRelationship = Query.SPATIAL_REL_INTERSECTS;
          }else
            return
          this.divResultMessage.textContent = "Searching Please wait...";
          var fields = ["*"];
          queryParams.returnGeometry = true;
          queryParams.outSpatialReference = this.map.spatialReference;
          queryParams.outFields = fields;
  
          var queryTask = new QueryTask(this.nearBySearchLayers[i]._nearByLayerUrl);
          defs.push(queryTask.execute(queryParams));
        }
        all(defs).then(lang.hitch(this, function (results) {
            this.log("results-found:" + results.length);
            var symUrl = this.folderUrl + "images/redball.png";
            var symbol = new PictureMarkerSymbol({
                "url" : symUrl, 
                "width" : 14, 
                "height" : 14, 
                "angle" : 0, 
                "xoffset" : 0, 
                "yoffset" : 0
            });
            array.forEach(results, lang.hitch(this, function (result, j) {
              this.log("featureType:" + j + "=" + result.geometryType);
              this.nearBySearchLayers[j].clear();
              for (var i = 0, len = result.features.length; i < len; i++) {//add the distance attribute
                var feature = result.features[i];
                feature._parentNearbyLayerName = this.nearBySearchLayers[j]._nearByLayerName;
                feature._performRevGeode = this.nearBySearchLayers[j]._nearByLayerRevGeocode;
                feature._parentNearbyLayerIndex = j;
                this.addDistanceAwayAttribute(feature);
                var useBall = this.config.useBallSymbol;
                if (useBall) {
                  feature.setSymbol(symbol);
                }else{ //use layers symbol or default symbol for the item list icon
                  if(feature.symbol === undefined){
                      if(this.nearBySearchLayers[j].renderer.defaultSymbol){
                        feature.setSymbol(this.nearBySearchLayers[j].renderer.defaultSymbol);
                      }else if(this.nearBySearchLayers[j].renderer.symbol){
                        feature.setSymbol(this.nearBySearchLayers[j].renderer.symbol);
                      }
                  }
                }
                this.nearBySearchLayers[j].add(feature);//adds it to particular search layer
                this.allFeaturesFound.push(feature);//save to list of all the features found no matter what layer
              }
              this.nearBySearchLayers[j].show();//show each individual feature layer
            }));
            this.createListOfItemsFromAllFeaturesForDisplay();
          }), lang.hitch(this, function (err) {
            console.log("nearby service layer call errors");
          })
        );
      },
      
      //once we have collected them all, create display list with all the features found from all the layers we searched
      createListOfItemsFromAllFeaturesForDisplay: function(){
         var lyrQLinks = [];//may populate in the future
         var listLen = 0;
         if (this.list.items.length > 0) {
             this.list.clear();//bug, having to do this here when coming from popuplink action
         }
         this.allFeaturesFound.sort(function(a,b){return a.attributes.MILES - b.attributes.MILES;});//sort all our features from all layers by distance
         for (var i = 0, len = this.allFeaturesFound.length; i < len; i++) {
           var content = "", rsltcontent = "", fldValue = "", oidVal, title = "";
           var feature = this.allFeaturesFound[i];
           this.addDistanceAwayAttribute(feature);
           var featureAttributes = feature.attributes;
           for (var att in featureAttributes) {
            var fld = att;
            var fldObjFromLyr = this.getField(this.nearBySearchLayers[feature._parentNearbyLayerIndex], fld);
            var fldValue = featureAttributes[att] !== null ? String(featureAttributes[att]) : "";
            fldValue = this.convertIfDate(fldObjFromLyr, fldValue,'MM/dd/yyyy');
            if (!this.fieldConfiguredForList(fld, feature._parentNearbyLayerIndex) && fld !== "MILES") {
              continue;//skip this field if not miles or configured for item list from config
            }
            content = content + this.resultFormatString.replace('[attribname]', fld).replace('[attribvalue]', fldValue);
            rsltcontent = rsltcontent + this.resultFormatString.replace('[attribname]',fld).replace('[attribvalue]', fldValue);
          }//end for, line item per featureAttribute 
          if (content.lastIndexOf('<br>') === (content.length - 4)) {
            content = content.substr(0, content.length - 4);
          }
          else {
            content = content;
          }
          if (rsltcontent.lastIndexOf('<br>') === (rsltcontent.length - 4)) {
            rsltcontent = rsltcontent.substr(0, rsltcontent.length - 4);
          }
          else {
            rsltcontent = rsltcontent;
          }
          var listItem = {
            id: "id_" + i + listLen,
            OID: oidVal,//not used, if needed grab it 
            title: feature._parentNearbyLayerName,
            content: content,
            rsltcontent: rsltcontent,
            alt: (i % 2 === 0),
            sym: feature.symbol,
            links: lyrQLinks,
            centerpoint: feature.geometry,
            graphic: feature,
            removeResultMsg: this.nls.removeResultMsg
          };
          this.list.add(listItem);
        }
        //html.setStyle(this.btnClear, 'display', 'block');
        if(this.list.items.length <= 0){
          this.divResultMessage.textContent = "No Results";
        }else{
          this.divResultMessage.textContent = this.list.items.length;
          if(this.config.countPrefix){
            this.divResultMessage.textContent = this.config.countPrefix + this.list.items.length;
          }
        }
      },
      
      convertIfDate: function(fldObj, fieldValue, dateformat){
        value = fieldValue !== null ? String(fieldValue) : "";
        if (value !== "") {
          var isDateField;
          if (fldObj) {
            isDateField = fldObj.type === "esriFieldTypeDate";
          }
          if (isDateField) {
            var dateMS = Number(fieldValue);
            if (!isNaN(dateMS)) {
                value = this.formatDate(dateMS, dateformat);
                return value;
            }
          }
        }
        return fieldValue;
      },
      
      formatDate: function (value, dateFormat) {
        if(dateFormat){
            dateFormat = dateFormat.replace(/D/g, "d").replace(/Y/g, "y");
        }
        var inputDate = new Date(value);
        return locale.format(inputDate, {
          selector: 'date',
          datePattern: dateFormat
        });
      },
      
      fieldConfiguredForList: function(fld,configLayerIndex){
        var configFieldArr = this.config.layers[configLayerIndex].fields.field;
        var configFieldItem;
        for (var i in configFieldArr) {
          configFieldItem = configFieldArr[i];
          if (configFieldItem.name.toLowerCase() === fld.toLowerCase() && configFieldItem.listitem) {
            return true;
          }
        }
        return false;
      },

			addDistanceAwayAttribute: function (g)
			{
				var g_worldgeo = webMercatorUtils.webMercatorToGeographic(g.geometry);
				var location_worldgeo = webMercatorUtils.webMercatorToGeographic(this.lastKnownGeometry);
				g.attributes.MILES = Number((this.distanceBetweenCoordinates(location_worldgeo.y, location_worldgeo.x,g_worldgeo.y,g_worldgeo.x)).toFixed(2));
			},
      
      //for non avl, not used 
      configurePopupTemplate: function (listItem) {
        var popUpInfo = {
          title: listItem.title,
          description: listItem.content,
          showAttachments: true
        };
        var pminfos = [];
        var popUpMediaInfo;

        for (var l = 0; l < listItem.links.length; l++) {
          if (listItem.links[l].link) {
            var pos = listItem.links[l].link.length - 4;
            var sfx = String(listItem.links[l].link).substr(pos, 4).toLowerCase();
            if (((sfx === ".jpg") || (sfx === ".png") || (sfx === ".gif")) && listItem.links[l].popuptype !== "text") {
              // use PopUpMediaInfo if it is an image
              if (!listItem.links[l].disableinpopup) {
                popUpMediaInfo = {};
                popUpMediaInfo.type = "image";
                var val = {};
                val.sourceURL = listItem.links[l].link;
                val.linkURL = listItem.links[l].link;
                popUpMediaInfo.value = val;
                popUpMediaInfo.caption = listItem.links[l].alias;
                pminfos.push(popUpMediaInfo);
              }
            } else if (listItem.links[l].icon !== "" && listItem.links[l].popuptype !== "text") {
              if (!listItem.links[l].disableinpopup) {
                popUpMediaInfo = {};
                popUpMediaInfo.type = 'image';
                popUpMediaInfo.value = {};
                popUpMediaInfo.value.sourceURL = listItem.links[l].icon;
                popUpMediaInfo.value.linkURL = listItem.links[l].link;
                popUpMediaInfo.caption = listItem.links[l].alias;
                pminfos.push(popUpMediaInfo);
              }
            } else {
              if (!listItem.links[l].disableinpopup) {
                var lText = (listItem.links[l].alias !== "") ? listItem.links[l].alias : listItem.links[l].link;
                popUpInfo.description += "<br><a href='" + listItem.links[l].link + "'>" + lText + "</a>";
              }
            }
          }
        }
        if (pminfos.length > 0) {
          popUpInfo.mediaInfos = pminfos;
        }
        //        console.info(popUpInfo);
        var pt = new PopupTemplate(popUpInfo);
        return pt;
      },
      
      
      returnListIndexFromOID: function (OID) {
        var retVal = -1;
        array.some(this.list.items, lang.hitch(this, function(item, index){
          if (item.OID === OID) {
            retVal = index;
            return true;
          }
        }));
        return retVal;
      },

      getAlias: function (att, layerIndex) {
        var field = this.config.layers[layerIndex].fields.field;
        var item;
        for (var i in field) {
          item = field[i];
          if (item.name.toLowerCase() === att.toLowerCase() && item.alias) {
            return item.alias;
          }
        }
        return att;
      },

      getField: function (layer, fieldName) {
        var result;
        if(fieldName === "MILES"){
          return {alias:"MILES",name:"MILES",type:"esriFieldTypeDouble"};
        }
        if (layer) {
          for (var f = 0; f < layer.fields.length; f++) {
            var field = layer.fields[f];
            if (fieldName == field.name) {
              result = field;
              break;
            }
          }
        }
        return result;
      },
      
      onSearchFinish: function (layerIndex, closeOnComplete, removing, adding, results) {
        this.log("onSearchFinish");
      },
      
      /**
      * This function create horizontal slider and set minimum maximum value of slider
      * @memberOf widgets/NHNearMe/Widget
      **/
      createSlider: function () {
        // initialize and set the parameter of slider
        this._slider = new HorizontalSlider({
          name: "slider",
          minimum: 0,
          maximum: this.config.maxBufferDistance,
          value: this.config.defaultBufferDistance,
          intermediateChanges: false,
          "class": "esriCTHorizantalSlider"
        }, this.horizantalSliderContainer);
        this._bufferParams = {
          BufferDistance: this._slider.value.toString(),
          BufferUnit: this.config.bufferDistanceUnit.label
        };
        // set slider content
        domAttr.set(this.silderText, "innerHTML", string.substitute(
          this.nls.bufferSliderText, this._bufferParams));
        // on change event of slider
        this.own(this._slider.on("change", lang.hitch(this, this.sliderChanged)));
        // set maximum and minimum value of horizontal slider
        this.sliderMinValue.innerHTML = this._slider.minimum.toString();
        this.sliderMaxValue.innerHTML = this._slider.maximum.toString();
      },
      
      sliderChanged: function(value){
        this.log("distSliderChanged");
        this._bufferParams.BufferDistance = value.toFixed(2);
        domAttr.set(this.silderText, "innerHTML", string.substitute(
        this.nls.bufferSliderText, this._bufferParams));
        if(this.searchWgt.value.length === 0) return;
        this.clear(false);
        this.highlightSelectedLocation(this.lastKnownGeometry);
        this.searchNearBy(this.lastKnownGeometry);
      },

			kmToFeet: function(kmN)
			{
				var feet=0;
				if(!kmN || isNaN(kmN))
					return feet;
				feet = kmN * 3280.84;
				return feet;
			},
      
			milesToKm: function(mi)
			{
				var kmOut=0;
				if(!mi || isNaN(mi))		
					return kmOut;
				kmOut = mi * 1.60934;
				return kmOut;
			},
			
			kmToMiles: function(kmN)
			{
				var miles=0;
				if(!kmN || isNaN(kmN))
					return miles;
				miles = kmN * .62137;
				return miles;
			},			
			
			distanceBetweenCoordinates: function(lat1,lon1,lat2,lon2, units)
			{
				var R = this.RADIUS_OF_EARTH_IN_MILES;
				if (units == "km"){R = this.RADIUS_OF_EARTH_IN_KM;}
				if (units == "meters"){R = this.RADIUS_OF_EARTH_IN_M;}
				if (units =="feet"){R=this.RADIUS_OF_EARTH_IN_FEET;}
				var dLat = (lat2-lat1) * Math.PI/180;
				var dLon = (lon2-lon1) * Math.PI/180;
				var lat1inRadians = lat1 * Math.PI/180;
				var lat2inRadians = lat2 * Math.PI/180;
				var a = Math.sin(dLat/2) * Math.sin(dLat/2) + Math.sin(dLon/2) * Math.sin(dLon/2) * Math.cos(lat1inRadians) * Math.cos(lat2inRadians);
				var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));
				var d = R * c;
				return d;
			},
      
			//Not used, param fixed Radius in kilometers
			createMercatorBasedCircle: function (fixedRadius,center,simpleFillSymbol)
			{	
				var lon1 = this.degToRad(center.x);  
				var lat1 = this.degToRad(center.y);  
				var R_KM = 6371; //radius km
				var R_MI = 3963;
				var d = fixedRadius/R_KM;  	//angular distance on earth's surface
				//var d:Number = fixedRadius/R_MI;  	//angular distance on earth's surface
				
				var nodes = 100; //number of nodes in circle
				var step = 360/nodes||10;
				var n = 0;
				var arrayOfPoints =[];	
				var m_circleGeometry = new Polygon(null, this.map.spatialReference);				
				
				for(var x = 0; x <= 360; x++)
				{	
					var z = Math.round(n+=step);
					var bearing = this.degToRad(x);
					
					var lat2 = Math.asin(
						Math.sin(lat1) * Math.cos(d) +
						Math.cos(lat1) * Math.sin(d) * Math.cos(bearing)
					);           
					
					var lon2 = lon1 + Math.atan2(
						Math.sin(bearing) * Math.sin(d) * Math.cos(lat1),
						Math.cos(d) - Math.sin(lat1) * Math.sin(lat2) 
					);				
					
					arrayOfPoints[x] = WebMercatorUtil.geographicToWebMercator( new MapPoint(this.radToDeg(lon2),this.radToDeg(lat2)));
				}						
				
				arrayOfPoints.push(arrayOfPoints[0]);    
				m_circleGeometry.addRing(arrayOfPoints);
				var circle = new Graphic(m_circleGeometry,simpleFillSymbol);			
				
				return circle			
				
			},
      		
      highlightSelectedLocation: function (locationGeometry) {
        var symbol;
        if (locationGeometry) {
          this.highlightGraphicsLayer.clear();
          // set the graphic symbol for selected center point on map
          symbol = symbolJsonUtils.fromJson(this.config.symbols.graphicLocationSymbol);
          //symbol.url = this.folderUrl + "images/locatorred.png";
          this.searchCenterPointGeo = locationGeometry;
          this.highlightGraphicsLayer.add(new Graphic(locationGeometry, symbol));
          this.highlightGraphicsLayer.show();
        }
      },

      //show buffer on map if buffer visibility is set to true in config
      showBuffer: function (bufferedGeometries) {
        if (this.config.bufferInfo && this.config.bufferInfo.isVisible) {
          this._bufferGraphicLayer.clear();
          if (this.config && this.config.symbols && this.config.symbols.bufferSymbol) {
            var symbol = symbolJsonUtils.fromJson(this.config.symbols.bufferSymbol);
            array.forEach(bufferedGeometries, lang.hitch(this, function (geometry) {
              var graphic = new Graphic(geometry, symbol);
              this._bufferGraphicLayer.add(graphic);
            }));
            this._bufferGraphicLayer.show();
          }
        }
      },
        
      degToRad: function (degrees) 
      {
      	return degrees * Math.PI / 180;
      },			
      radToDeg: function (radians)
      {
      	return radians * 180 / Math.PI;
      },
      
      onSearchError: function (error) {
        this.log("onSearchError");
        console.log(error);
      },

      timerEvent: function() {
        this.m_iTimerTicks++;
      },
      
      log: function(messageString){
        if(this.DEBUG){
          console.log(messageString);
        }
      },
      

      onReceiveData: function(name, widgetId, data, historyData) {
        this.log("NHNearMe-OnReceiveData:" + data.message + ":value:" + data.searchvalue);
        this.searchMessage = null;
        if(name !== 'Search'){
          return;
        }
        
        this.searchMessage = data.searchvalue;
        if (this.searchWgt && this.searchMessage && data.message == "SearchToNHNearMe") {
               this.log("NearMeonReceiveData-Searching");
               this.clearButtonClick();
               this.searchWgt.set('value', this.searchMessage);
               this.searchWgt.search();
        }
      },


    });
    return clazz;
  });